<?php
// +----------------------------------------------------------------------
// +----------------------------------------------------------------------
// | Author: star xiong <875376798@qq.com>
// | QQ: 875376798
// +----------------------------------------------------------------------

namespace app\admin\traits;

use app\common\model\ModelModel;
use app\common\model\CategoryModel;
use app\common\model\ContentModel;
use app\common\model\AttributeModel;
use app\common\model\ContentAttributeModel;
use app\common\model\CategoryTypeModel;
use app\common\model\CategoryAttributeModel;
use app\common\model\ContentGroupModel;
use app\common\model\ContentTagModel;
use app\common\model\ContentTopModel;
use app\admin\service\TreeService;
use think\facade\Cache;
use think\Db;

trait ContentTrait
{
	private function save($param, $mid) {
		Db::startTrans();
		try{

			//是否发布
			if (isset($param['status']) && $param['status'] == 'on') {
				$param['status'] = 1;
			} else {
				 $param['status'] = 0;
			}
			
			//是否开新窗口
			if (isset($param['target']) && $param['target'] == 'on') {
				$param['target'] = 1;
			} else {
				$param['target'] = 0;
			}
			
			if(!isset($param['istop'])){
				$param['istop'] = 0;
			}
			
			//标签tag
			if($param['tag']){
				$param['tag'] = trim($param['tag'],' ');
				$param['tag'] = trim($param['tag'],',');
				//$param['tag'] = trim($param['tag'],'，');
				$param['tag'] = str_replace(' ','',$param['tag']);
				$param['tag'] = str_replace('；',',',$param['tag']);
				$param['tag'] = str_replace(';',',',$param['tag']);
				$param['tag'] = str_replace('，',',',$param['tag']);				
			}
			
			//群组group
			if($param['group']){
				$param['group'] = trim($param['group'],' ');
				$param['group'] = trim($param['group'],',');
				//$param['group'] = trim($param['group'],'，');
				$param['group'] = str_replace(' ','',$param['group']);
				$param['group'] = str_replace('；',',',$param['group']);
				$param['group'] = str_replace(';',',',$param['group']);
				$param['group'] = str_replace('，',',',$param['group']);
			}

			//处理多图/图组/复选框
			foreach($param as $key => $field) {
				if (is_array($field)) {
					$param[$key] = implode(',', $field);
				}else{
					$param[$key] = trim($field);
				}
			}
			
			$param['siteid'] = session('site');
			
			$modelModel = new ModelModel();
			$tableName = $modelModel->get($mid);
			$table_name = "form_".$tableName['table_name'];
			
			$param['content_id'] = Db::name($table_name)->strict(false)->insertGetId($param);
			
			$result = false;
			if ($param['content_id']) {
				$now_time = date("Y-m-d H:m:s");
				$param['created_at'] = $now_time;
				$param['update_at'] = $now_time;
				$user = session($this->loginUserKey);
				$param['uid'] = $user['id'];
				
				$param['mid'] = $mid;
				$contentModel = new ContentModel();
				$result = $contentModel->strict(false)->insertGetId($param);
				
				if($result){
					//推荐
					$tops = explode(',', $param['istop']);
					if(is_array($tops)) {
						$content_tops = [];
						foreach($tops as $key=>$top){
							if(!empty($top)) {
								$ct = [];
								$ct['content_id'] = $result;
								$ct['top_id'] = $top;
								$content_tops[] = $ct;
							}
							
						}
						if(!empty($content_tops)){
							$contentTopModel = new ContentTopModel();
							$contentTopModel->saveAll($content_tops);
						}
					}

					//标签tag
					
					$tags = array_unique(explode(',', $param['tag']));
					$content_tags = [];
					foreach($tags as $key=>$tag){
						if(!empty($tag)) {
							$tg = [];
							$tg['content_id'] = $result;
							$tg['tag'] = $tag;
							$content_tags[] = $tg;
						}
						
					}
					if(!empty($content_tags)){
						$contentTagModel = new ContentTagModel();
						$contentTagModel->saveAll($content_tags);
					}
					
					//群组group
					$groups = explode(',', $param['group']);
					$content_groups = [];
					foreach($groups as $key=>$group){
						if(!empty($group)) {
							$gp = [];
							$gp['content_id'] = $result;
							$gp['group_id'] = $group;
							$content_groups[] = $gp;
						}
						
					}
					if(!empty($content_groups)) {
						$contentGroupModel = new ContentGroupModel();
						$contentGroupModel->saveAll($content_groups);
					}
					
					//属性
					$attributeModel = new AttributeModel();
					$list = $attributeModel->All();
					$attributes = [];
					foreach($list as $K=>$V){
						$attributes[$V['field']] = $V['id'];
					}
					$attrDate = [];
					foreach($param as $key => $field) {
						if(substr($key, 0, 5) === 'attr_') {
							$attr = [];
							$attr['content_id'] = $result;
							$attr['attr_id'] = $attributes[$key];
							$attr['attr_value'] = trim($field);
							$attrDate[] = $attr;
						}
					}
					if(!empty($attrDate)) {
						$contentAttributeModel = new ContentAttributeModel();
						$flag = $contentAttributeModel->saveAll($attrDate);
					}
				}
			}
			
			Db::commit();
			
			if(false === $result){
				 Db::rollback();
				return ['code' => 1, 'data' => '', 'msg' => '添加内容失败'];
			}else{
				return ['code' => 0, 'data' => $result, 'msg' => '添加内容成功'];
			}
			
		}catch( PDOException $e){
			Db::rollback();
			return ['code' => 2, 'data' => '', 'msg' => $e->getMessage()];
		}
	}
	
	private function update($param) {
		
		Db::startTrans();
		try{
			$id = intval($param['id']);
			unset($param['id']);
			
			$contentModel = new ContentModel();
			$content = $contentModel->get($id);
			$modelModel = new ModelModel();
			$model = $modelModel->get($content['mid']);
			$table_name = "form_".$model['table_name'];

			//是否发布
			if (isset($param['status']) && $param['status'] == 'on') {
				$param['status'] = 1;
			} else {
				 $param['status'] = 0;
			}
			
			//是否开新窗口
			if (isset($param['target']) && $param['target'] == 'on') {
				$param['target'] = 1;
			} else {
				 $param['target'] = 0;
			}
			
			if(!isset($param['istop'])){
				$param['istop'] = 0;
			}
			
			//推荐
			$contentTopModel = new ContentTopModel();
			$contentTopModel->where('content_id', $id)->delete();
			if(is_array($param['istop'])){
				$content_tops = [];
				foreach($param['istop'] as $key=>$top){
					if(!empty($top)) {
						$ct = [];
						$ct['content_id'] = $id;
						$ct['top_id'] = $top;
						$content_tops[] = $ct;
					}
					
				}
				if(!empty($content_tops)){
					$contentTopModel->saveAll($content_tops);
				}
			}
			
			//标签tag
			$contentTagModel = new ContentTagModel();
			$contentTagModel->where('content_id', $id)->delete();
			if($param['tag']){

				$param['tag'] = rtrim($param['tag'],' ');
				$param['tag'] = trim($param['tag'],',');
				//$param['tag'] = trim($param['tag'],'，');
				$param['tag'] = str_replace(' ','',$param['tag']);
				$param['tag'] = str_replace('；',',',$param['tag']);
				$param['tag'] = str_replace(';',',',$param['tag']);
				$param['tag'] = str_replace('，',',',$param['tag']);

				$tags = $tags = array_unique(explode(',', $param['tag']));
				$content_tags = [];
				foreach($tags as $key=>$tag){
					if(!empty($tag)) {
						$tg = [];
						$tg['content_id'] = $id;
						$tg['tag'] = $tag;
						$content_tags[] = $tg;
					}
				}
				
				if(!empty($content_tags)){
					$contentTagModel->saveAll($content_tags);
				}
			}

			//群组group
			$contentGroupModel = new ContentGroupModel();
			$contentGroupModel->where('content_id', $id)->delete();
			if(!empty($param['group'])){
				$param['group'] = trim($param['group'],' ');
				$param['group'] = trim($param['group'],',');
				//$param['group'] = trim($param['group'],'，');
				$param['group'] = str_replace(' ','',$param['group']);
				$param['group'] = str_replace('；',',',$param['group']);
				$param['group'] = str_replace(';',',',$param['group']);
				$param['group'] = str_replace('，',',',$param['group']);
				
				$groups = explode(',', $param['group']);
				$content_groups = [];
				foreach($groups as $key=>$group){
					if(!empty($group)) {
						$gp = [];
						$gp['content_id'] = $id;
						$gp['group_id'] = $group;
						$content_groups[] = $gp;
					}
				}

				if(!empty($content_groups)){
					$contentGroupModel->saveAll($content_groups);
				}
			}

			//属性
			$attributeModel = new AttributeModel();
			$contentAttributeModel = new ContentAttributeModel();

			$list = $attributeModel->All();
			$attributes = [];
			foreach($list as $K=>$V){
				$attributes[$V['field']] = $V['id'];
			}
			
			
			
			$attrDate = [];
			foreach($param as $key => $field) {
				//处理多图/图组/复选框
				if(is_array($field)){
					$param[$key] = implode(',', $field);
					//unset($fields[''])
				}
				else{
					$param[$key] = trim($field);
				}
				
				if(substr($key, 0, 5) === 'attr_') {					
					$attr = [];
					$attr['content_id'] = $id;
					$attr['attr_id'] = $attributes[$key];
					$attr['attr_value'] = $param[$key];
					$attrDate[] =$attr;
					unset($param[$key]);
				}
			}
			
			//自定义字段中checkbox字段无值提交时，设值为空
			$fields = $model->fields->where('class','checkbox')->column('field');
			foreach($fields as $field){
				if(empty($param[$field])){
					$param[$field] = '';
				}
			}
			
			$contentAttributeModel->where('content_id', $id)->delete();
			if(!empty($attrDate)) {
				$contentAttributeModel->saveAll($attrDate);
			}
			
			//更新content
			
			Db::name($table_name)->strict(false)
								->where('id', $content['content_id'])
								->update($param);
								
			if(empty($param['update_at'])) {
				$param['update_at'] = date("Y-m-d H:m:s");
			}

			//更新content表
			$contentModel->strict(false)
						->where('id', $id)
						->update($param);
			Db::commit();
			return ['code' => 0, 'data' => '', 'msg' => '内容更新成功'];
			
		}catch( PDOException $e){
			Db::rollback();
			return ['code' => 2, 'data' => '', 'msg' => $e->getMessage()];
		}
	}
	
	/**
	 * update category and page
	 *
	 * @time at 2019年03月26日
	 * @param $params
	 * @return \think\Paginator
	 */
	private function updateCatAndPage($param) {
		
		Db::startTrans();
		try{
			
			$modelModel = new ModelModel();
			
			$model = $modelModel->get($param['mid']);			
			$table_name = "form_".$model['table_name'];

			
			//是否显示
			if (isset($param['status']) && $param['status'] == 'on') {
				$param['status'] = 1;
			} else {
				 $param['status'] = 0;
			}
			
			//是否允许发布内容 栏目属性
			if (isset($param['type']) && $param['type'] == 'on') {
				$param['type'] = 1;
			} else {
				 $param['type'] = 0;
			}
			
			//是否开新窗口
			if (isset($param['target']) && $param['target'] == 'on') {
				$param['target'] = 1;
			} else {
				 $param['target'] = 0;
			}
			
			foreach($param as $key => $field) {
				if (is_array($field)) {
					$param[$key] = implode(',', $field);
				}
				
			}
			
			//单页面
			if($model['type'] == 2) {
				$id = $param['id'];
				
				unset($param['id']);
				
				if($param['page_id']) {
					Db::name($table_name)->strict(false)
										->where('id', $param['page_id'])
										->update($param);
				} else {				
					$param['page_id'] = Db::name($table_name)->strict(false)->insertGetId($param);
					
				}
				$param['id'] = $id;
			}
			
			$param['types'] = explode(',', $param['types']);
			$param['attributes'] = explode(',', $param['attributes']);
	
			//栏目类型
			$types = $param['types'];
			unset($param['types']);
			
			//栏目属性
			$attributes = $param['attributes'];
			unset($param['attributes']);

			$categoryModel = new CategoryModel();
			
			$list[] = $param;
			$categoryModel->saveAll($list);
			
			//栏目类型			
			$cateTypeModel = new CategoryTypeModel();
			$cateTypeModel->saveTypes($types,$param['id'],true);
			
			//栏目属性
			$categoryAttributeModel = new CategoryAttributeModel();
			$categoryAttributeModel->saveAttributes($attributes, $param['id'], true);
			
			Db::commit();
			
			//缓存栏目
			$this->categorys_catch();
			
			return ['code' => 0, 'data' => '', 'msg' => '内容更新成功'];
			
		}catch( PDOException $e){
			Db::rollback();
			return ['code' => 2, 'data' => '', 'msg' => $e->getMessage()];
		}
	}
	
	private function del($id)
	{
		Db::startTrans();
		try{
			$contentModel = new ContentModel();
			$content = $contentModel->get($id);
			
			$modelModel = new ModelModel();
			$model = $modelModel->get($content['mid']);
			
			$table_name = "form_".$model['table_name'];
			
			$falg = $content->delete($id);
			
			Db::name($table_name)->delete($content['content_id']);
			
			//删除推荐top
			$contentTopModel = new ContentTopModel();
			$contentTopModel->where('content_id', $id)->delete();
			//删除标签tag
			$contentTagModel = new ContentTagModel();
			$contentTagModel->where('content_id', $id)->delete();
			//删除群组group
			$contentGroupModel = new ContentGroupModel();
			$contentGroupModel->where('content_id', $id)->delete();
			
			
			Db::commit();
			
			return ['code'=> 0, 'msg'=>'删除数据成功', 'data'=> $falg];
			 
		}catch( PDOException $e){
			Db::rollback();
			return ['code' => 2, 'data' => '', 'msg' => $e->getMessage()];
		}
	}
	
	/**
	 * get content
	 *
	 * @time at 2019年03月25日
	 * @param $params
	 * @return \think\Paginator
	 */
	private function getContentFull($id)
	{
		$contentModel = new ContentModel();
		$contentAttributeModel = new ContentAttributeModel();
		$attributeModel = new AttributeModel();
		$aList = $attributeModel->column('field','id');
		$content = $contentModel->get($id);
		$caList = $contentAttributeModel->where('content_id', $id)->select();
		$contentAttributes = [];
		foreach($caList as $key=>$value){
			$contentAttributes[$aList[$value['attr_id']]] = $value['attr_value'];
		}
		$table_name = "form_".$content->model->table_name;
		$more_info = Db::name($table_name)->get($content['content_id']);
		$content['more'] = $more_info;
		$content['attributes'] = $contentAttributes;
		return $content;
	}
	
	/**
	 * get page
	 *
	 * @time at 2019年03月26日
	 * @param $params
	 * @return \think\Paginator
	 */
	private function getPage($mid, $page_id)
	{
		
		$modelModel = new ModelModel();
		
		$model = $modelModel->get($mid);
		
		$table_name = "form_".$model['table_name'];
		
		$page = Db::name($table_name)->get($page_id);

		return $page;
	}
	
	//缓存栏目
	private function categorys_catch() {
		$treeService = new TreeService();
		$categoryModel = new CategoryModel();
		$categorys = $categoryModel->where('siteid', session('site'))
								->where('status', 1)
								->with(['model','typelist','attrlist'])
								->order("sort","acs")
								->select();
		Cache::connect(config('cate_cache_options'))->set(session('site'), $treeService->tree($categorys));
	}
}
