<?php
namespace database;
use think\Db;
use Hook;
class Field 
{
	public $connect =false;
    function __construct($config=array())  //析构函数
    {
		 
         if(count($config) > 0)
		 {
			    $this ->config= $config; 
			    $this -> db = Db::connect($config);
		 }
		 else
		 {     
		       $this ->config= config('database.'); 
			   $this -> db = Db::connect($config);
		 }
    } 
	
/*************************************************************
	get_field 获取数据表中的字段
	@param   table string   字段所在数据表的名称
	@return  array 
*************************************************************/
    public function get_field($table)
    {
           switch ($this ->config['type'])
		   {
					case 'mysql':
					  return $this ->mysql_get_field($table);
					  break;  
					default:
					  return $this ->mysql_get_field($table);
		   }
    }
	
/*************************************************************
	get_field 获取数据表中的字段
	@param   table string   字段所在数据表的名称
	@return  int 
*************************************************************/
    public function mysql_get_field($table)
    {
		   if(!$table) return [];
		   
		   $table = $this ->config['prefix'].$table;
		   $info = $this -> db -> query('SELECT * FROM information_schema.Columns WHERE table_name="'.$table.'" AND table_schema="'.$this ->config['database'].'"');
		   $returnData =[];
		   foreach($info as $v)
		   {
			    $v = array_change_key_case($v);
			    $returnData[$v['column_name']] = array_change_key_case($v);
		   }
		   
		   return $returnData;
    }
	
/*************************************************************
	field_to_array 将数据表中的字段转变成字段信息记录
	@param   table string   字段所在数据表的名称
	@return  array 
*************************************************************/
    public function field_to_array($table)
    {
           switch ($this ->config['type'])
		   {
					case 'mysql':
					  return $this ->mysql_field_to_array($table);
					  break;  
					default:
					  return $this ->mysql_field_to_array($table);
		   }
    }
	
/*************************************************************
	get_field MYSQL将数据表中的字段转变成字段信息记录
	@param   table string   字段所在数据表的名称
	@return  int 
*************************************************************/
    public function mysql_field_to_array($table)
    {
		   if(!$table) return [];
		   $field_array = $this->mysql_get_field($table);
		   
		   $returnData=[];
		   foreach($field_array as $v)
		   {
					$data=[];
					$data['type']=$v["data_type"];
					$data['field']=$v["column_name"];
					$data['table']=["table_name"];
					$data['database']=["table_schema"];
					$data['is_null']=strtolower($v["column_name"])=="yes" ? 1 : 0;
					$data['m']=$v["numeric_precision"] && is_numeric($v["numeric_precision"]) && $v["numeric_precision"]>0 ? $v["numeric_precision"] : 11;
					$data['d']=$v["numeric_scale"] && is_numeric($v["numeric_scale"]) && $v["numeric_scale"]>0 ? $v["numeric_scale"] : 0;
					$data['unsigned']=$v["column_type"] && strpos($v["column_type"],"unsigned")>0 ? 1 : 0;
					$data['describe']=$v["describe"] ? $v["describe"] : "";
			
		   }
		   
    }		
	
/*************************************************************
	has 检测数据表中是否存在字段
	@param  field string  字段名称
	@param  table string  表名称
	@return  int 
*************************************************************/
    public function has($field,$table)
    {
           switch ($this ->config['type'])
		   {
				  case 'mysql':
					return $this ->mysql_has($field,$table);
					break;  
				  default:
					return $this ->mysql_has($field,$table);
		   }
    }
	
/*************************************************************
	mysql_has 检测MYSQL检测数据表中是否存在字段
	@param  field string  字段名称
	@param  table string  表名称
	@return  int 
*************************************************************/
    public function mysql_has($field,$table)
    {
		  $table = $this -> config['prefix'].$table;
		  $check = $this -> db ->query('SELECT * FROM information_schema.Columns WHERE column_name = "'.$field.'" AND table_name="'.$table.'" AND table_schema="'.$this ->config['database'].'"');	 
		  return $check ? 1 : 0;

    }
	
/*************************************************************
	field_create 创建字段
	@param  fieldInfo array  字段信息
	@return  int 
*************************************************************/
    public function field_create($fieldInfo=[])
    {
           switch ($this ->config['type'])
		   {
				  case 'mysql':
					return $this ->mysql_field_create($fieldInfo);
					break;  
				  default:
					return $this ->mysql_field_create($fieldInfo);
		   }
    }
	
/*************************************************************
	mysql_field_create 创建MYSQL字段
	@param  fieldInfo array  字段信息
	@return  int  0:创建失败，1:创建成功
*************************************************************/
    public function mysql_field_create($fieldInfo=[])
    {
		   if(count($fieldInfo) <=0) return 0;
		   
		   $field = isset($fieldInfo['field']) && $fieldInfo['field'] ? $fieldInfo['field'] : NULL;
		   $table = isset($fieldInfo['table']) && $fieldInfo['table'] ? $fieldInfo['table'] : NULL;
		   $type = isset($fieldInfo['type']) && $fieldInfo['type'] ? strtolower($fieldInfo['type']) : "varchar";
		   
		   
		   $comment = isset($fieldInfo['comment']) ? " COMMENT  '".$fieldInfo['comment']."' " : "";
		   $is_null = isset($fieldInfo['is_null']) && $fieldInfo['is_null'] ? " NULL " : " NOT NULL ";

		   if(!$field || !$table) return 0;
		   
		   try{
				    
					if($this->mysql_has($field,$table)) return 1;
					
					switch ($type)
					{
						case "varchar":
						      $default = isset($fieldInfo['default']) ? " DEFAULT  '".$fieldInfo['default']."' " : "";
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` ADD `".$field."` VARCHAR(255) ".$is_null.$default.$comment;
							  break;  
						case "int":
						case "tinyint":
						case "smallint":
						case "bigint":
						      $unsigned = isset($fieldInfo['unsigned']) && $fieldInfo['unsigned'] ? "UNSIGNED "  : "";
						      $default = isset($fieldInfo['default']) ? (is_numeric($fieldInfo['default']) ? " DEFAULT  ".$fieldInfo['default'] : " DEFAULT 0 ") : "";
						      $m = isset($fieldInfo['m']) && $fieldInfo['m'] ? $fieldInfo['m'] : 11;
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` ADD `".$field."` ".$type."(".$m.")".$unsigned.$is_null.$default.$comment;
							  break;
						case "float":
						case "decimal":
						case "double":
						      $unsigned = isset($fieldInfo['unsigned']) && $fieldInfo['unsigned'] ? "UNSIGNED "  : "";
						      $default = isset($fieldInfo['default']) ? " DEFAULT  ".$fieldInfo['default'] : "";
							  $m = isset($fieldInfo['m']) && $fieldInfo['m'] ? $fieldInfo['m'] : 11;
							  $d = isset($fieldInfo['d']) && $fieldInfo['d'] ? $fieldInfo['d'] : 0; 
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` ADD `".$field."` ".$type."(".$m.",".$d.")".$unsigned.$is_null.$default.$comment;
							  break;
						case "text":
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` ADD `".$field."` text ".$is_null.$comment;
							  break;
						default:
						      $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` ADD `".$field."` ".$type." ".$comment;
					}

					$create = $this -> db ->execute($sql.";");		
					return $this->mysql_has($field,$table) ? 1 : 0;
			}catch(\Exception $e){
				
				   return 0;
			}

    }

/*************************************************************
	field_edit 编辑字段
	@param  fieldInfo array  字段信息
	@return  int 
*************************************************************/
    public function field_edit($fieldInfo=[])
    {
           switch ($this ->config['type'])
		   {
				  case 'mysql':
					return $this ->mysql_field_edit($fieldInfo);
					break;  
				  default:
					return $this ->mysql_field_edit($fieldInfo);
		   }
    }
	
/*************************************************************
	mysql_field_edit 编辑mysql字段
	@param  fieldInfo array  字段信息
	@return  int   0:编辑失败，1:编辑成功
*************************************************************/
    public function mysql_field_edit($fieldInfo=[])
    {
		   if(count($fieldInfo) <=0) return 0;
		   
		   $field = isset($fieldInfo['field']) && $fieldInfo['field'] ? $fieldInfo['field'] : NULL;
		   $table = isset($fieldInfo['table']) && $fieldInfo['table'] ? $fieldInfo['table'] : NULL;
		   $type = isset($fieldInfo['type']) && $fieldInfo['type'] ? strtolower($fieldInfo['type']) : "varchar";
		   
		   
		   $comment = isset($fieldInfo['comment']) ? " COMMENT  '".$fieldInfo['comment']."' " : "";
		   $is_null = isset($fieldInfo['is_null']) && $fieldInfo['is_null'] ? " NULL " : " NOT NULL ";

		   if(!$field || !$table) return 0;
		   
		   try{
				    
					if(!$this->mysql_has($field,$table)) return $this->field_create($fieldInfo);
					
					switch ($type)
					{
						case "varchar":
						      $default = isset($fieldInfo['default']) ? " DEFAULT  '".$fieldInfo['default']."' " : "";
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` MODIFY `".$field."` VARCHAR(255) ".$is_null.$default.$comment;
							  break;  
						case "int":
						case "tinyint":
						case "smallint":
						case "bigint":
						      $unsigned = isset($fieldInfo['unsigned']) && $fieldInfo['unsigned'] ? "UNSIGNED "  : "";
						      $default = isset($fieldInfo['default']) ? (is_numeric($fieldInfo['default']) ? " DEFAULT  ".$fieldInfo['default'] : " DEFAULT 0 ") : "";
						      $m = isset($fieldInfo['m']) && $fieldInfo['m'] ? $fieldInfo['m'] : 11;
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` MODIFY `".$field."` ".$type."(".$m.")".$unsigned.$is_null.$default.$comment;
							  break;
						case "float":
						case "decimal":
						case "double":
						      $unsigned = isset($fieldInfo['unsigned']) && $fieldInfo['unsigned'] ? "UNSIGNED "  : "";
						      $default = isset($fieldInfo['default']) ? " DEFAULT  ".$fieldInfo['default'] : "";
							  $m = isset($fieldInfo['m']) && $fieldInfo['m'] ? $fieldInfo['m'] : 11;
							  $d = isset($fieldInfo['d']) && $fieldInfo['d'] ? $fieldInfo['d'] : 0; 
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` MODIFY `".$field."` ".$type."(".$m.",".$d.")".$unsigned.$is_null.$default.$comment;
							  break;
						case "text":
							  $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` MODIFY `".$field."` text ".$is_null.$comment;
							  break;
						default:
						      $sql = "ALTER TABLE `".$this ->config['prefix'].$table."` MODIFY `".$field."` ".$type." ".$comment;
					}

					$edit = $this -> db ->execute($sql.";");		
					return $edit!==false ? 1 : 0;
			}catch(\Exception $e){
				   return 0;
			}

    }
	
/*************************************************************
	field_del 删除字段
	@param  fieldInfo array  字段信息
	@return  int 
*************************************************************/
    public function field_del($fieldInfo=[])
    {
           switch ($this ->config['type'])
		   {
				  case 'mysql':
					return $this ->mysql_field_del($fieldInfo);
					break;  
				  default:
					return $this ->mysql_field_del($fieldInfo);
		   }
    }
	
/*************************************************************
	mysql_field_edit 编辑mysql字段
	@param  fieldInfo array  字段信息
	@return  int  -1标识字段不存在 0:编辑失败，1:编辑成功
*************************************************************/
    public function mysql_field_del($fieldInfo=[])
    {
		   if(count($fieldInfo) <=0) return 0;
		   
		   $field = isset($fieldInfo['field']) && $fieldInfo['field'] ? $fieldInfo['field'] : NULL;
		   $table = isset($fieldInfo['table']) && $fieldInfo['table'] ? $fieldInfo['table'] : NULL;

		   if(!$field || !$table) return 0;
		   
		   try{
				    
					if(!$this->mysql_has($field,$table)) return 1;
					
					$sql = "alter table `".$this ->config['prefix'].$table."` drop column `".$field."`";

					$del = $this -> db ->execute($sql.";");		
					return !$this->mysql_has($field,$table) ? 1 : 0;
			}catch(\Exception $e){
				   return 0;
			}

    }
}  	
	
					
?>