<?php
/**
 * Permissions.php
 * Created by wuyanwen <wuyanwen1992@gmail.com>
 * Date: 2018/9/26 0026 20:12
 */
namespace think\permissions\model;

use think\Model;

class Permissions extends Model
{
	public function initialize()
	{
		parent::initialize(); // TODO: Change the autogenerated stub
		$this->name = config('permissions.table.permission');
	}

	/**
	 * 关联权限
	 *
	 * @time at 2018年09月21日
	 * @return \think\model\relation\BelongsToMany
	 */
	public function roles()
	{
		return $this->belongsToMany(config('permissions.table.permission'), 'role_has_permissions', 'role_id', 'permission_id');
	}

	/**
	 * find by permission by id
	 *
	 * @author wuyanwen <wuyanwen1992@gmail.com> at 2018年09月28日 23:27
	 * @param $permission_id
	 * @return null|static
	 */
	public function getPermissionBy($permission_id)
	{
		return self::get($permission_id);
	}

	/**
	 * Delete Permission
	 *
	 * @author wuyanwen <wuyanwen1992@gmail.com> at 2018年09月28日 23:28
	 * @param $permission_id
	 * @return bool|int
	 */
	public function deleteBy($permission_id)
	{
		return $this->getPermissionBy($permission_id)->delete();
	}

	/**
	 * update
	 *
	 * @author wuyanwen <wuyanwen1992@gmail.com> at 2018年09月28日 23:31
	 * @param $id
	 * @param $data
	 * @return static
	 */
	public function updateBy(int $id, array $data)
	{
		return self::where('id', $id)->update($data);
	}

	/**
	 * store
	 *
	 * @author wuyanwen <wuyanwen1992@gmail.com> at 2018年09月28日 23:32
	 * @param array $data
	 * @return bool
	 */
	public function store(array $data)
	{
		$this->data($data);

		return $this->save();
	}

	/**
	 * Find Permission By Module & Controller & Action
	 *
	 * @time at 2018年09月29日
	 * @param $module
	 * @param $controller
	 * @param $action
	 * @return array|false|\PDOStatement|string|\think\Model
	 */
	public function getPermissionByModuleAnd($module, $controller, $action)
	{
		return $this->where([
			'module'     => $module,
			'controller' => $controller,
			'action'     => $action,
		])->find();
	}

	/**
	 * 删除 权限相关的角色
	 *
	 * @author wuyanwen <wuyanwen1992@gmail.com> at 2018年10月02日 22:01
	 * @param $permission_id
	 * @return int
	 */
	public function detachRole($permission_id)
	{
		return $this->getPermissionBy($permission_id)->roles()->detach();
	}
}
